@@ -3,19 +3,18 @@
  *
  * Robert Love <rml@ximian.com>
  *
+ * gnome-volume-manager is a simple policy engine that implements a state
+ * machine in response to events from HAL.  Responding to these events,
+ * gnome-volume-manager implements automount, autorun, autoplay, automatic
+ * photo management, and so on.
+ *
  * Licensed under the GNU GPL v2.  See COPYING.
  *
  * (C) Copyright 2004 Novell, Inc.
- *
- * gnome-volume-manager is a simple policy agent that sits on top of the stack
- * of system-level components including the kernel, hotplug, udev, and HAL.
- * Responding to HAL events, gnome-volume-manager implements automount,
- * autorun, autoplay, automatic photo management, and so on.
  */
 
 #include "config.h"
 
-#include <string.h>
 #include <gnome.h>
 #include <gconf/gconf-client.h>
 #include <gdk/gdkx.h>
@@ -26,20 +25,19 @@
 #include "gvm.h"
 
 #ifdef ENABLE_NLS
-#  include <libintl.h>
-#  define _(String) gettext (String)
-#  ifdef gettext_noop
-#    define N_(String) gettext_noop (String)
-#  else
-#    define N_(String) (String)
-#  endif
+# include <libintl.h>
+# define _(String) gettext (String)
+# ifdef gettext_noop
+#   define N_(String) gettext_noop (String)
+# else
+#   define N_(String) (String)
+# endif
 #else
-#  define _(String)
-#  define N_(String) (String)
+# define _(String)
+# define N_(String) (String)
 #endif
 
-#define GVM_DEBUG
-
+//#define GVM_DEBUG
 #ifdef GVM_DEBUG
 # define dbg(fmt,arg...) fprintf(stderr, "%s/%d: " fmt,__FILE__,__LINE__,##arg)
 #else
@@ -49,8 +47,10 @@
 #define warn(fmt,arg...) g_warning("%s/%d: " fmt,__FILE__,__LINE__,##arg)
 
 #define BIN_MOUNT	"/bin/mount"	/* what we mount with */
+#define NAUTILUS_COMMAND "/usr/bin/nautilus -n --no-desktop %m"
 
-struct gvm_configuration config;
+static struct gvm_configuration config;
+static LibHalContext *hal_ctx;
 
 /*
  * gvm_load_config - synchronize gconf => config structure
@@ -64,6 +64,8 @@ gvm_load_config (void)
 			GCONF_ROOT "automount_media", NULL);
 	config.autoplay_cda = gconf_client_get_bool (config.client,
 			GCONF_ROOT "autoplay_cda", NULL);
+	config.autobrowse = gconf_client_get_bool (config.client,
+			GCONF_ROOT "autobrowse", NULL);
 	config.autorun = gconf_client_get_bool(config.client,
 			GCONF_ROOT "autorun", NULL);
 	config.autophoto = gconf_client_get_bool(config.client,
@@ -82,13 +84,16 @@ gvm_load_config (void)
 			GCONF_ROOT "autoburn_cdr_command", NULL);
 	config.autophoto_command = gconf_client_get_string (config.client,
 			GCONF_ROOT "autophoto_command", NULL);
+	config.eject_command = gconf_client_get_string (config.client,
+			GCONF_ROOT "eject_command", NULL);
 
 	/*
 	 * If all of the options that control our policy are disabled, then we
 	 * have no point in living.  Save the user some memory and exit.
 	 */
-	if (!(config.automount_drives || config.autorun || config.autoplay_cda
-			|| config.autoplay_dvd || config.autophoto)) {
+	if (!(config.automount_drives || config.autobrowse || config.autorun 
+			|| config.autoplay_cda 	|| config.autoplay_dvd 
+			|| config.autophoto)) {
 		dbg ("daemon exit: no point living\n");
 		exit (EXIT_SUCCESS);
 	}
@@ -98,22 +103,18 @@ gvm_load_config (void)
  * gvm_config_changed - gconf_client_notify_add () call back to reload config
  */
 static void
-gvm_config_changed (GConfClient *client, guint id,
-		    GConfEntry *entry, gpointer data)
+gvm_config_changed (GConfClient *client __attribute__((__unused__)),
+		    guint id __attribute__((__unused__)),
+		    GConfEntry *entry __attribute__((__unused__)),
+		    gpointer data __attribute__((__unused__)))
 {
-	(void) client;
-	(void) id;
-	(void) entry;
-	(void) data;
-
-	dbg ("Configuration: changed!\n");
-
 	g_free (config.autoplay_cda_command);
 	g_free (config.autorun_path);
 	g_free (config.autoplay_dvd_command);
 	g_free (config.autoburn_cdr_command);
 	g_free (config.autophoto_command);
-
+	g_free (config.eject_command);
+    
 	gvm_load_config ();
 }
 
@@ -158,8 +159,7 @@ gvm_run_command (const char *device, const char *command, const char *path)
 			g_string_append (exec, device);
 			q = p + 2;
 			p = p + 2;
-		}
-		if (*(p + 1) == 'm') {
+		} else if (*(p + 1) == 'm') {
 			*p = '\0';
 			g_string_append (exec, q);
 			g_string_append (exec, path);
@@ -254,6 +254,9 @@ gvm_check_dvd (const char *device, const char *mount_point)
  * ask the user if he wants to import the photos.
  *
  * Returns TRUE if there were photos on this device, FALSE otherwise
+ *
+ * FIXME: Should probably not prompt the user and just do it automatically.
+ *        This now makes sense, as gphoto added an import mode.
  */
 static gboolean
 gvm_check_photos (const char *udi, const char *device, const char *mount_point)
@@ -271,9 +274,9 @@ gvm_check_photos (const char *udi, const char *device, const char *mount_point)
 	retval = TRUE;
 	dbg ("Photos detected: %s\n", dcim_path);
 
-	/* add the "camera" capability to this device */
-	if (!hal_device_add_capability (udi, "camera"))
-		warn ("failed to add camera capability to %s\n", device);
+	/* add the "content.photos" capability to this device */
+	if (!hal_device_add_capability (hal_ctx, udi, "content.photos"))
+		warn ("failed to set content.photos on %s\n", device);
 
 	if (config.autophoto) {
 		askme = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_QUESTION,
@@ -293,7 +296,6 @@ gvm_check_photos (const char *udi, const char *device, const char *mount_point)
 
 out:
 	g_free (dcim_path);
-
 	return retval;
 }
 
@@ -306,16 +308,18 @@ static void
 gvm_device_autorun (const char *udi)
 {
 	char *device = NULL, *mount_point = NULL;
+	gboolean autorun_succeeded = FALSE;
 
-	device = hal_device_get_property_string (udi, "block.device");
+	device = hal_device_get_property_string (hal_ctx, udi, "block.device");
 	if (!device) {
 		warn ("cannot get block.device\n");
 		goto out;
 	}
 
-	mount_point = hal_device_get_property_string (udi, "block.mount_point");
+	mount_point = hal_device_get_property_string (hal_ctx, udi, 
+						      "volume.mount_point");
 	if (!mount_point) {
-		warn ("cannot get block.mount_point\n");
+		warn ("cannot get volume.mount_point\n");
 		goto out;
 	}
 
@@ -350,6 +354,9 @@ gvm_device_autorun (const char *udi)
 				if (error)
 					warn ("failed to exec %s: %s\n", path,
 					      error->message);
+				else
+					autorun_succeeded = TRUE;
+				
 				g_free (path);
 				break;
 			}
@@ -359,6 +366,10 @@ gvm_device_autorun (const char *udi)
 
 		g_strfreev (autorun_fns);
 	}
+	
+	if ((config.autobrowse == TRUE) && (autorun_succeeded == FALSE)) {
+		gvm_run_command (device, NAUTILUS_COMMAND, mount_point);
+	}
 
 out:
 	hal_free_string (device);
@@ -415,7 +426,7 @@ gvm_ask_mixed (const char *udi)
 	enum { MOUNT, PLAY } action = -1;
 	char *device = NULL, *mount_point = NULL;
 
-	device = hal_device_get_property_string (udi, "block.device");
+	device = hal_device_get_property_string (hal_ctx, udi, "block.device");
 	if (!device) {
 		warn ("cannot get block.device\n");
 		goto out;
@@ -460,11 +471,10 @@ out:
  * given device node, if so configured
  */
 static void
-gvm_run_cdburner (const char *device, const char *mount_point)
+gvm_run_cdburner (const char *device, const char *mount)
 {
 	if (config.autoburn_cdr)
-		gvm_run_command (device, config.autoburn_cdr_command,
-				 mount_point);
+		gvm_run_command (device, config.autoburn_cdr_command, mount);
 }
 
 /*
@@ -473,16 +483,15 @@ gvm_run_cdburner (const char *device, const char *mount_point)
 static gboolean
 gvm_device_is_writer (const char *udi)
 {
-	if (hal_device_get_property_int (udi, "storage.cdr") == 1)
-		return 1;
-	else if (hal_device_get_property_int (udi, "storage.cdrw") == 1)
-		return 1;
-	else if (hal_device_get_property_int (udi, "storage.dvdr") == 1)
-		return 1;
-	else if (hal_device_get_property_int (udi, "storage.dvdram") == 1)
-		return 1;
+	if ((hal_device_get_property_bool (hal_ctx, udi, "storage.cdrom.cdr")) ||
+	    (hal_device_get_property_bool (hal_ctx, udi, "storage.cdrom.cdrw")) ||
+	    (hal_device_get_property_bool (hal_ctx, udi, "storage.cdrom.dvdr")) ||
+	    (hal_device_get_property_bool (hal_ctx, udi, "storage.cdrom.dvdram")) ||
+	    (hal_device_get_property_bool (hal_ctx, udi, "storage.cdrom.dvdplusr")) ||
+	    (hal_device_get_property_bool (hal_ctx, udi, "storage.cdrom.dvdplusrw")))
+		return TRUE;
 
-	return 0;
+	return FALSE;
 }
 
 /*
@@ -492,35 +501,44 @@ gvm_device_is_writer (const char *udi)
 static void
 gvm_cdrom_policy (const char *udi)
 {
-	char *type, *device = NULL;
+	char *device = NULL;
+	char *drive_udi = NULL;
+	dbus_bool_t has_audio;
+	dbus_bool_t has_data;
+	dbus_bool_t is_blank;
 
-	type = hal_device_get_property_string (udi, "storage.cdrom.media_type");
-	if (!type) {
-		warn ("unable to get storage.cdrom.media_type for %s\n", udi);
-		goto out;
-	}
+	has_audio = hal_device_get_property_bool (hal_ctx, udi,
+						  "volume.disc.has_audio");
+	has_data = hal_device_get_property_bool (hal_ctx, udi,
+						  "volume.disc.has_data");
+	is_blank = hal_device_get_property_bool (hal_ctx, udi,
+						  "volume.disc.is_blank");
+	drive_udi = hal_device_get_property_string(hal_ctx, udi,
+			"info.parent");
 
-	device = hal_device_get_property_string (udi, "block.device");
+	device = hal_device_get_property_string (hal_ctx, udi, "block.device");
 	if (!device) {
 		warn ("cannot get block.device\n");
 		goto out;
 	}
 
-	if (!strcmp (type, "audio")) {
+	if (has_audio && (!has_data)) {
 		gvm_run_cdplayer (device, device);
-	} else if (!strcmp (type, "mixed")) {
+	} else if (has_audio && has_data) {
 		gvm_ask_mixed (udi);
-	} else if (!strcmp (type, "data")) {
+	} else if (has_data) {
 		if (config.automount_media)
 			gvm_device_mount (device);
-	} else if (!strcmp (type, "blank")) {
-		if (gvm_device_is_writer (device))
+	} else if (is_blank) {
+		if (gvm_device_is_writer (drive_udi))
 			gvm_run_cdburner (device, device);
 	}
 
+	/** @todo enforce policy for all the new disc types now supported */
+
 out:
 	hal_free_string (device);
-	hal_free_string (type);
+	hal_free_string (drive_udi);
 }
 
 /*
@@ -539,9 +557,10 @@ gvm_media_changed (const char *udi)
 	char *media_type;
 
 	/* get HAL's interpretation of our media type */
-	media_type = hal_device_get_property_string (udi, "storage.media");
+	media_type = hal_device_get_property_string (hal_ctx, udi, 
+						     "storage.drive_type");
 	if (!media_type) {
-		warn ("cannot get storage.media\n");
+		warn ("cannot get storage.drive_type\n");
 		return;
 	}
 
@@ -553,253 +572,200 @@ gvm_media_changed (const char *udi)
 	hal_free_string (media_type);
 }
 
-/*
- * handle_device_added - handle HAL's DeviceAdded signal
+/** Invoked when a device is added to the Global Device List. 
  *
- * This is where we learn about new device's, so this is where we perform
- * user-configurable policy such as auto-mounting.
+ *  @param  ctx                 LibHal context
+ *  @param  udi                 Universal Device Id
  */
-static DBusHandlerResult
-handle_device_added (DBusMessage *message)
+static void
+hal_device_added (LibHalContext *ctx __attribute__((__unused__)), 
+		  const char *udi)
 {
-	char *udi;
-	DBusError error;
+	char *device = NULL, *storage_device = NULL;
 
-	dbus_error_init (&error);
-
-	if (dbus_message_get_args (message, &error,
-				   DBUS_TYPE_STRING, &udi,
-				   DBUS_TYPE_INVALID)) {
-		char *bus, *device = NULL, *storage_device = NULL;
-
-		bus = hal_device_get_property_string (udi, "info.bus");
-		if (!bus) {
-			warn ("cannot get info.bus\n");
-			goto out;
-		}
-
-		dbg ("New Device: %s\n", udi);
-
-		/* is this a block device ? */
-		if (g_strcasecmp (bus, "block"))
-			goto out;
-
-		/* is this a mountable volume ? */
-		if (!hal_device_get_property_bool (udi, "block.is_volume"))
-			goto out;
-
-		/* if it is a volume, it must have a device node */
-		device = hal_device_get_property_string (udi, "block.device");
-		if (!device) {
-			dbg ("cannot get block.device\n");
-			goto out;
-		}
-
-		/* get the backing storage device */
-		storage_device = hal_device_get_property_string (udi,
-				"block.storage_device");
-		if (!storage_device) {
-			dbg ("cannot get block.storage_device\n");
-			goto out;
-		}
-
-		/*
-		 * Does this device support removable media?  Note that we
-		 * check storage_device and not our own UDI
-		 */
-		if (hal_device_get_property_bool (storage_device,
-						  "storage.removable")) {
-			/* we handle media change events separately */
-			dbg ("Changed: %s\n", device);
-			gvm_media_changed (udi);
-			goto out;
-		}
-
-		/* folks, we have a new device! */
-		dbg ("Added: %s\n", device);
-
-		if (config.automount_drives)
-			gvm_device_mount (device);
+	dbg ("New Device: %s\n", udi);
 
+	if (!hal_device_query_capability(hal_ctx, udi, "block"))
+		goto out;
+	
+	/* is this a mountable volume ? */
+	if (!hal_device_get_property_bool (hal_ctx, udi, 
+					   "block.is_volume"))
+		goto out;
+	
+	/* if it is a volume, it must have a device node */
+	device = hal_device_get_property_string (hal_ctx, udi, 
+						 "block.device");
+	if (!device) {
+		dbg ("cannot get block.device\n");
+		goto out;
+	}
+	
+	/* get the backing storage device */
+	storage_device = hal_device_get_property_string (
+		hal_ctx, udi,
+		"block.storage_device");
+	if (!storage_device) {
+		dbg ("cannot get block.storage_device\n");
+		goto out;
+	}
+	
+	/*
+	 * Does this device support removable media?  Note that we
+	 * check storage_device and not our own UDI
+	 */
+	if (hal_device_get_property_bool (hal_ctx, storage_device,
+					  "storage.removable")) {
+		/* we handle media change events separately */
+		dbg ("Changed: %s\n", device);
+		gvm_media_changed (udi);
+		goto out;
+	}
+	
+	/* folks, we have a new device! */
+	dbg ("Added: %s\n", device);
+	
+	if (config.automount_drives)
+		gvm_device_mount (device);
+	
 out:
-		hal_free_string (bus);
-		hal_free_string (device);
-		hal_free_string (storage_device);
-	} else
-		dbg ("dbus_message_get_args failed to parse message\n");
-
-	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+	hal_free_string (device);
+	hal_free_string (storage_device);
 }
 
-/*
- * handle_device_removed - respond to HAL's DeviceRemoved signal
+/** Invoked when a device is removed from the Global Device List. 
  *
- * We currently do not do anything special here, except print out debug
- * messages.  In the future, this is where we will learn about new devices.
+ *  @param  ctx                 LibHal context
+ *  @param  udi                 Universal Device Id
  */
-static DBusHandlerResult
-handle_device_removed (DBusMessage *message)
+static void
+hal_device_removed (LibHalContext *ctx __attribute__((__unused__)), 
+		    const char *udi)
 {
-	char *udi;
-	DBusError error;
-
-	dbus_error_init (&error);
-
-	if (dbus_message_get_args (message, &error,
-				   DBUS_TYPE_STRING, &udi,
-				   DBUS_TYPE_INVALID))
-		dbg ("Device removed: %s\n", udi);
-	else
-		dbg ("dbus_message_get_args failed to parse message\n");
-
-	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+	dbg ("Device removed: %s\n", udi);
 }
 
-/*
- * handle_property_modify - respond to HAL's PropertyModified signal
+/** Invoked when device in the Global Device List acquires a new capability.
  *
- * We listen to this bad-boy to learn about change in mount status.  We handle
- * auto-run as a response to auto-mount, which is pretty cunning.
+ *  @param  ctx                 LibHal context
+ *  @param  udi                 Universal Device Id
+ *  @param  capability          Name of capability
  */
-static DBusHandlerResult
-handle_property_modify (DBusMessage *message)
+static void
+hal_device_new_capability (LibHalContext *ctx __attribute__((__unused__)),
+			   const char *udi __attribute__((__unused__)), 
+			   const char *capability __attribute__((__unused__)))
 {
-	dbus_bool_t added, removed;
-	DBusMessageIter iter;
-	dbus_int32_t nr;
-	const char *udi;
-	char *key;
-	int i;
+}
 
-	udi = dbus_message_get_path (message);
+/** Invoked when device in the Global Device List loses a capability.
+ *
+ *  @param  ctx                 LibHal context
+ *  @param  udi                 Universal Device Id
+ *  @param  capability          Name of capability
+ */
+static void
+hal_device_lost_capability (LibHalContext *ctx __attribute__((__unused__)),
+			    const char *udi __attribute__((__unused__)), 
+			    const char *capability __attribute__((__unused__)))
+{
+}
 
-	dbus_message_iter_init (message, &iter);
-	nr = dbus_message_iter_get_int32 (&iter);
-	dbus_message_iter_next (&iter);
+/** Invoked when a property of a device in the Global Device List is
+ *  changed, and we have we have subscribed to changes for that device.
+ *
+ *  @param  ctx                 LibHal context
+ *  @param  udi                 Univerisal Device Id
+ *  @param  key                 Key of property
+ */
+static void
+hal_property_modified (LibHalContext *ctx __attribute__((__unused__)),
+		       const char *udi, 
+		       const char *key,
+		       dbus_bool_t is_removed __attribute__((__unused__)), 
+		       dbus_bool_t is_added __attribute__((__unused__)))
+{
+	if (!g_strcasecmp (key, "volume.is_mounted")) {
+		dbus_bool_t val;
+		
+		val = hal_device_get_property_bool (hal_ctx, udi, key);
+		if (val == TRUE) {
+			dbg ("Mounted: %s\n", udi);
+			gvm_device_autorun (udi);
+		} else
+			dbg ("Unmounted: %s\n", udi);
+	}	
+}
 
-	for (i = 0; i < nr; i++) {
-		key = dbus_message_iter_get_string (&iter);
-		dbus_message_iter_next (&iter);
-		removed = dbus_message_iter_get_boolean (&iter);
-                dbus_message_iter_next (&iter);
-                added = dbus_message_iter_get_boolean (&iter);
-                dbus_message_iter_next (&iter);
 
-		if (!g_strcasecmp (key, "block.is_mounted")) {
-			dbus_bool_t val;
+/** Invoked when a device in the GDL emits a condition that cannot be
+ *  expressed in a property (like when the processor is overheating)
+ *
+ *  @param  ctx                 LibHal context
+ *  @param  udi                 Univerisal Device Id
+ *  @param  condition_name      Name of condition
+ *  @param  message             D-BUS message with parameters
+ */
+static void
+hal_device_condition (LibHalContext *ctx __attribute__((__unused__)),
+		      const char *udi __attribute__((__unused__)), 
+		      const char *condition_name __attribute__((__unused__)),
+		      DBusMessage * message __attribute__((__unused__)))
+{
+	if (!g_strcasecmp (condition_name, "EjectPressed")) {		
+		char *argv[3];
+		GError *error = NULL;
+		char *device;
 
-			val = hal_device_get_property_bool (udi, key);
-			if (val == TRUE) {
-				dbg ("Mounted: %s\n", udi);
-				gvm_device_autorun (udi);
-			} else
-				dbg ("Unmounted: %s\n", udi);
+		device = hal_device_get_property_string (ctx, udi,
+							 "block.device");
+		if (!device)
+			warn ("cannot get block.device\n");
+		else if (config.eject_command) {
+			argv[0] = config.eject_command;
+			argv[1] = device;
+			argv[2] = NULL;
+
+			g_spawn_async (g_get_home_dir (), argv, NULL, 0, NULL,
+				       NULL, NULL, &error);
+			if (error)
+				warn ("failed to exec %s: %s\n",
+				      config.eject_command, error->message);
 		}
 
-		dbus_free (key);
+		hal_free_string (device);
 	}
-
-	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
 
-#define MAN_DBUS_OBJ		"org.freedesktop.Hal.Manager"
-#define DEV_DBUS_OBJ		"org.freedesktop.Hal.Device"
-#define DBUS_DBUS_OBJ		"org.freedesktop.DBus"
-#define DEVICE_ADDED_SIG	"DeviceAdded"
-#define DEVICE_REMOVED_SIG	"DeviceRemoved"
-#define PROPERTY_MOD_SIG	"PropertyModified"
 
-/*
- * dbus_filter - generic filter handler for the signals we listen on
- */
-static DBusHandlerResult
-dbus_filter (DBusConnection *connection,
-	     DBusMessage *message, gpointer user_data)
-{
-	(void) connection;
-	(void) user_data;
-
-	if (dbus_message_is_signal (message, MAN_DBUS_OBJ, DEVICE_ADDED_SIG))
-		return handle_device_added (message);
-
-	if (dbus_message_is_signal (message, MAN_DBUS_OBJ, DEVICE_REMOVED_SIG))
-		return handle_device_removed (message);
-
-	if (dbus_message_is_signal (message, DEV_DBUS_OBJ, PROPERTY_MOD_SIG))
-		return handle_property_modify (message);
-
-	dbg ("unhandled dbus message: %s from %s\n",
-	     dbus_message_get_member (message),
-	     dbus_message_get_sender (message));
-
-	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
-}
-
-/*
- * gvm_init_dbus - initialize D-BUS and listen to the appropriate signals
+/** Invoked by libhal for integration with our mainloop. 
  *
- * Returns TRUE is successful and FALSE if any step fails.  Since we cannot
- * continue without a successful D-BUS setup, a FALSE return value is fatal.
+ *  @param  ctx                 LibHal context
+ *  @param  dbus_connection     D-BUS connection to integrate
  */
-static gboolean
-gvm_init_dbus (void)
+static void
+hal_mainloop_integration (LibHalContext *ctx __attribute__((__unused__)),
+			  DBusConnection * dbus_connection)
 {
-	DBusConnection *bus;
-	DBusError error;
-
-	dbus_error_init (&error);
-
-	/* get on the system bus */
-	bus = dbus_bus_get_with_g_main (DBUS_BUS_SYSTEM, 0);
-	if (!bus) {
-		dbg ("cannot connect to system bus: %s\n", error.message);
-		dbus_error_free (&error);
-		return FALSE;
-	}
-
-#define HAL_MANAGER_RULE "type='signal',interface='org.freedesktop.Hal.Manager',path='/org/freedesktop/Hal/Manager'"
-	dbus_bus_add_match (bus, HAL_MANAGER_RULE, &error);
-	if (dbus_error_is_set (&error)) {
-		dbg ("failed to add match: %s\n", error.message);
-		dbus_error_free (&error);
-		return FALSE;
-	}
-
-#define HAL_DEVICE_RULE "type='signal',interface='org.freedesktop.Hal.Device'"
-	dbus_bus_add_match (bus, HAL_DEVICE_RULE, &error);
-	if (dbus_error_is_set (&error)) {
-		dbg ("failed to add match: %s\n", error.message);
-		dbus_error_free (&error);
-		return FALSE;
-	}
-
-#define KERNEL_RULE "type='signal',interface='org.kernel.drivers.cdrom.change',path='/org/kernel/drivers/cdrom/change'"
-	dbus_bus_add_match (bus, KERNEL_RULE, &error);
-	if (dbus_error_is_set (&error)) {
-		dbg ("failed to add match: %s\n", error.message);
-		dbus_error_free (&error);
-		return FALSE;
-	}
-
-	if (!dbus_connection_add_filter (bus, dbus_filter, NULL, NULL)) {
-		dbg ("failed to add filter\n");
-		return FALSE;
-	}
-
-	dbus_connection_flush (bus);
-
-	return TRUE;
+	dbus_connection_setup_with_g_main (dbus_connection, NULL);
 }
 
+
 int
 main (int argc, char *argv[])
 {
 	GnomeClient *client;
+	LibHalFunctions hal_functions = { hal_mainloop_integration,
+					  hal_device_added,
+					  hal_device_removed,
+					  hal_device_new_capability,
+					  hal_device_lost_capability,
+					  hal_property_modified,
+					  hal_device_condition };
 
 	gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE,
 			    argc, argv, GNOME_PARAM_NONE);
 
-
 	bindtextdomain(PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset(PACKAGE, "UTF-8");
 	textdomain(PACKAGE);
@@ -816,13 +782,14 @@ main (int argc, char *argv[])
 	gtk_signal_connect (GTK_OBJECT (client), "die",
 			    GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
 
-	if (gvm_init_dbus () == FALSE) {
-		warn ("failed to initialize D-BUS!\n");
+	hal_ctx = hal_initialize (&hal_functions, FALSE);
+	if (!hal_ctx) {
+		warn ("failed to initialize HAL!\n");
 		return 1;
 	}
 
-	if (hal_initialize (NULL, FALSE)) {
-		warn ("failed to initialize HAL!\n");
+	if (hal_device_property_watch_all (hal_ctx)) {
+		warn ("failed to watch all HAL properties!\n");
 		return 1;
 	}
 
