@@ -1,7 +1,7 @@
 /*
  * src/properties.c - control panel applet for gnome-volume-manager
  *
- * Robert Love <rml@ximian>
+ * Robert Love <rml@ximian.com>
  *
  * (C) Copyright 2004 Novell, Inc.
  *
@@ -34,6 +34,7 @@
 GtkWidget *dialog;
 GtkWidget *automount_drives_cb;
 GtkWidget *automount_media_cb;
+GtkWidget *autobrowse_cb;
 GtkWidget *autorun_cb;
 GtkWidget *autoplay_cda_cb;
 GtkWidget *command_cda_hbox;
@@ -81,6 +82,8 @@ write_config (void)
 			       config.automount_drives, NULL);
 	gconf_client_set_bool (config.client, GCONF_ROOT "automount_media",
 			       config.automount_media, NULL);
+	gconf_client_set_bool (config.client, GCONF_ROOT "autobrowse",
+			       config.autobrowse, NULL);
 	gconf_client_set_bool (config.client, GCONF_ROOT "autorun",
 			       config.autorun, NULL);
 	gconf_client_set_bool (config.client, GCONF_ROOT "autoburn_cdr",
@@ -114,6 +117,8 @@ signal_response (void)
 			(automount_drives_cb)->active;
 	config.automount_media = GTK_TOGGLE_BUTTON
 			(automount_media_cb)->active;
+	config.autobrowse = GTK_TOGGLE_BUTTON
+			(autobrowse_cb)->active;
 	config.autorun = GTK_TOGGLE_BUTTON (autorun_cb)->active;
 	config.autoburn_cdr = GTK_TOGGLE_BUTTON (autoburn_cdr_cb)->active;
 	config.autoplay_cda = GTK_TOGGLE_BUTTON (autoplay_cda_cb)->active;
@@ -163,10 +168,10 @@ check_daemon (void)
 	char *argv[2];
 	GError *error = NULL;
 
-	if (!(config.automount_drives || config.automount_media
-			|| config.autorun || config.autoplay_cda
-			|| config.autoplay_dvd || config.autophoto)
-			|| check_clipboard ()) 
+	if (!(config.automount_drives || config.automount_media 
+			|| config.autobrowse || config.autorun 
+			|| config.autoplay_cda || config.autoplay_dvd 
+			|| config.autophoto) || check_clipboard ()) 
 		return;
 
 	argv[0] = BINDIR "/gnome-volume-manager";
@@ -201,6 +206,8 @@ update_widgets (void)
 				      config.automount_drives);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (automount_media_cb),
 				      config.automount_media);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (autobrowse_cb),
+				      config.autobrowse);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (autorun_cb),
 				      config.autorun);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (autoplay_cda_cb),
@@ -242,6 +249,8 @@ read_config (void)
 			GCONF_ROOT "automount_drives", NULL);
 	config.automount_media = gconf_client_get_bool (config.client,
 			GCONF_ROOT "automount_media", NULL);
+	config.autobrowse = gconf_client_get_bool (config.client,
+			GCONF_ROOT "autobrowse", NULL);
 	config.autorun = gconf_client_get_bool (config.client,
 			GCONF_ROOT "autorun", NULL);
 	config.autoplay_cda = gconf_client_get_bool (config.client,
@@ -274,14 +283,11 @@ read_config (void)
  * us to reread our configuration.
  */
 static void
-config_change_cb (GConfClient *client, guint id, GConfEntry *entry,
-		   gpointer data)
+config_change_cb (GConfClient *client __attribute__((__unused__)),
+		  guint id __attribute__((__unused__)),
+		  GConfEntry *entry __attribute__((__unused__)),
+		  gpointer data __attribute__((__unused__)))
 {
-	(void) client;
-	(void) id;
-	(void) entry;
-	(void) data;
-
 	read_config ();
 }
 
@@ -383,6 +389,7 @@ init_capplet (void)
 
 	automount_drives_cb = glade_xml_get_widget (xml, "automount_drives_cb");
 	automount_media_cb = glade_xml_get_widget (xml, "automount_media_cb");
+	autobrowse_cb = glade_xml_get_widget (xml, "autobrowse_cb");
 	autorun_cb = glade_xml_get_widget (xml, "autorun_cb");
 
 	autoplay_cda_cb = glade_xml_get_widget (xml, "autoplay_cda_cb");
@@ -427,12 +434,13 @@ init_capplet (void)
 			FALSE);
 
 	read_config ();
-	update_widgets ();
 
 	gtk_signal_connect (GTK_OBJECT (automount_drives_cb), "toggled",
 			    GTK_SIGNAL_FUNC (signal_response), NULL);
 	gtk_signal_connect (GTK_OBJECT (automount_media_cb), "toggled",
 			    GTK_SIGNAL_FUNC (signal_response), NULL);
+	gtk_signal_connect (GTK_OBJECT (autobrowse_cb), "toggled",
+			    GTK_SIGNAL_FUNC (signal_response), NULL);
 	gtk_signal_connect (GTK_OBJECT (autorun_cb), "toggled",
 			    GTK_SIGNAL_FUNC (signal_response), NULL);
 
